@@ -1,6 +1,6 @@
 from typing import Optional
 
-from rsocket.frame_helpers import ensure_bytes, safe_len
+from rsocket.frame_helpers import safe_len
 from rsocket.local_typing import ByteTypes
 
 
@@ -22,8 +22,8 @@ class Payload:
         self._check(data)
         self._check(metadata)
 
-        self.data = ensure_bytes(data)
-        self.metadata = ensure_bytes(metadata)
+        self.data = data
+        self.metadata = metadata
 
     def __str__(self):
         return f"<payload: data_length {safe_len(self.data)}, metadata_length {safe_len(self.metadata)}>"
