@@ -1,13 +1,13 @@
 from dataclasses import dataclass
-from typing import Optional
+from typing import Optional, Union, Callable
 
-from reactivex import Observable, Observer
+from reactivex import Observable, Observer, Subject
 
 from rsocket.frame import MAX_REQUEST_N
 
 
 @dataclass(frozen=True)
 class ReactivexChannel:
-    observable: Optional[Observable] = None
+    observable: Optional[Union[Observable, Callable[[Subject], Observable]]] = None
     observer: Optional[Observer] = None
     limit_rate: int = MAX_REQUEST_N
